<?php

/*
 phpFlickr
 * Written by Dan Coulter (dan@dancoulter.com)
 * Project Home Page: http://github.com/dancoulter/phpflickr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
if ( ! class_exists( 'phpFlickr' ) ) {
	if ( session_id() == '' ) {
		@session_start();
	}

	class phpFlickr {

		var $api_key;
		var $secret;
		var $rest_endpoint    = 'https://api.flickr.com/services/rest/';
		var $upload_endpoint  = 'https://up.flickr.com/services/upload/';
		var $replace_endpoint = 'https://up.flickr.com/services/replace/';
		var $req;
		var $response;
		var $parsed_response;
		var $last_request = null;
		var $die_on_error;
		var $error_code;
		var $error_msg;
		var $token;
		var $php_version;
		var $custom_post = null;

		function __construct( $api_key, $secret = null, $die_on_error = false ) {
			// The API Key must be set before any calls can be made.  You can
			// get your own at https://www.flickr.com/services/api/misc.api_keys.html
			$this->api_key      = $api_key;
			$this->secret       = $secret;
			$this->die_on_error = $die_on_error;
			$this->service      = 'flickr';

			// Find the PHP version and store it for future reference
			$this->php_version = explode( '-', phpversion() );
			$this->php_version = explode( '.', $this->php_version[0] );
		}

		function setCustomPost( $function ) {
			$this->custom_post = $function;
		}

		function post( $data, $type = null ) {
			if ( is_null( $type ) ) {
				$url = $this->rest_endpoint;
			}

			if ( ! is_null( $this->custom_post ) ) {
				return call_user_func( $this->custom_post, $url, $data );
			}

			if ( ! preg_match( '|https://(.*?)(/.*)|', $url, $matches ) ) {
				die( 'There was some problem figuring out your endpoint' );
			}

			if ( function_exists( 'curl_init' ) ) {
				// Has curl. Use it!
				$curl = curl_init( $this->rest_endpoint );
				curl_setopt( $curl, CURLOPT_POST, true );
				curl_setopt( $curl, CURLOPT_POSTFIELDS, $data );
				curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true );
				$response = curl_exec( $curl );
				curl_close( $curl );
			}
			return $response;
		}

		function request( $command, $args = array(), $nocache = false ) {
			// Sends a request to Flickr's REST endpoint via POST.
			if ( substr( $command, 0, 7 ) != 'flickr.' ) {
				$command = 'flickr.' . $command;
			}

			// Process arguments, including method and login data.
			$args = array_merge(
				array(
					'method'         => $command,
					'format'         => 'json',
					'nojsoncallback' => '1',
					'api_key'        => $this->api_key,
				),
				$args
			);
			if ( ! empty( $this->token ) ) {
				$args = array_merge( $args, array( 'auth_token' => $this->token ) );
			} elseif ( ! empty( $_SESSION['phpFlickr_auth_token'] ) ) {
				$args = array_merge( $args, array( 'auth_token' => $_SESSION['phpFlickr_auth_token'] ) );
			}
			ksort( $args );
			$auth_sig           = '';
			$this->last_request = $args;
			foreach ( $args as $key => $data ) {
				if ( is_null( $data ) ) {
					unset( $args[ $key ] );
					continue;
				}
				$auth_sig .= $key . $data;
			}
			if ( ! empty( $this->secret ) ) {
				$api_sig         = md5( $this->secret . $auth_sig );
				$args['api_sig'] = $api_sig;
			}
			$this->response = $this->post( $args );

			/*
			 * Uncomment this line (and comment out the next one) if you're doing large queries
			 * and you're concerned about time.  This will, however, change the structure of
			 * the result, so be sure that you look at the results.
			 */
			$this->parsed_response = json_decode( $this->response, true );
			/* 			$this->parsed_response = $this->clean_text_nodes(json_decode($this->response, TRUE)); */
			if ( $this->parsed_response['stat'] == 'fail' ) {
				if ( $this->die_on_error ) {
					die( "The Flickr API returned the following error: #{$this->parsed_response['code']} - {$this->parsed_response['message']}" );
				} else {
					$this->error_code      = $this->parsed_response['code'];
					$this->error_msg       = $this->parsed_response['message'];
					$this->parsed_response = false;
				}
			} else {
				$this->error_code = false;
				$this->error_msg  = false;
			}
			return $this->response;
		}

		function clean_text_nodes( $arr ) {
			if ( ! is_array( $arr ) ) {
				return $arr;
			} elseif ( count( $arr ) == 0 ) {
				return $arr;
			} elseif ( count( $arr ) == 1 && array_key_exists( '_content', $arr ) ) {
				return $arr['_content'];
			} else {
				foreach ( $arr as $key => $element ) {
					$arr[ $key ] = $this->clean_text_nodes( $element );
				}
				return( $arr );
			}
		}


		function getErrorCode() {
			// Returns the error code of the last call.  If the last call did not
			// return an error. This will return a false boolean.
			return $this->error_code;
		}

		function getErrorMsg() {
			// Returns the error message of the last call.  If the last call did not
			// return an error. This will return a false boolean.
			return $this->error_msg;
		}

		/* These functions are front ends for the flickr calls */

		function buildPhotoURL( $photo, $size = 'Medium' ) {
			// receives an array (can use the individual photo data returned
			// from an API call) and returns a URL (doesn't mean that the
			// file size exists)
			$sizes = array(
				'square'     => '_s',
				'square_75'  => '_s',
				'square_150' => '_q',
				'thumbnail'  => '_t',
				'small'      => '_m',
				'small_240'  => '_m',
				'small_320'  => '_n',
				'medium'     => '',
				'medium_500' => '',
				'medium_640' => '_z',
				'medium_800' => '_c',
				'large'      => '_b',
				'large_1024' => '_b',
				'large_1600' => '_h',
				'large_2048' => '_k',
				'original'   => '_o',
			);

			$size = strtolower( $size );
			if ( ! array_key_exists( $size, $sizes ) ) {
				$size = 'medium';
			}

			if ( $size == 'original' ) {
				$url = 'https://farm' . $photo['farm'] . '.static.flickr.com/' . $photo['server'] . '/' . $photo['id'] . '_' . $photo['originalsecret'] . '_o' . '.' . $photo['originalformat'];
			} else {
				$url = 'https://farm' . $photo['farm'] . '.static.flickr.com/' . $photo['server'] . '/' . $photo['id'] . '_' . $photo['secret'] . $sizes[ $size ] . '.jpg';
			}
			return $url;
		}

		function auth( $perms = 'read', $remember_uri = true ) {
			// Redirects to Flickr's authentication piece if there is no valid token.
			// If remember_uri is set to false, the callback script (included) will
			// redirect to its default page.

			if ( empty( $_SESSION['phpFlickr_auth_token'] ) && empty( $this->token ) ) {
				if ( $remember_uri == true ) {
					$_SESSION['phpFlickr_auth_redirect'] = $_SERVER['REQUEST_URI'];
				} elseif ( $remember_uri != false ) {
					$_SESSION['phpFlickr_auth_redirect'] = $remember_uri;
				}
				$api_sig = md5( $this->secret . 'api_key' . $this->api_key . 'perms' . $perms );

				if ( $this->service == '23' ) {
					header( 'Location: http://www.23hq.com/services/auth/?api_key=' . $this->api_key . '&perms=' . $perms . '&api_sig=' . $api_sig );
				} else {
					header( 'Location: https://www.flickr.com/services/auth/?api_key=' . $this->api_key . '&perms=' . $perms . '&api_sig=' . $api_sig );
				}
				exit;
			} else {
				$tmp                = $this->die_on_error;
				$this->die_on_error = false;
				$rsp                = $this->auth_checkToken();
				if ( $this->error_code !== false ) {
					unset( $_SESSION['phpFlickr_auth_token'] );
					$this->auth( $perms, $remember_uri );
				}
				$this->die_on_error = $tmp;
				return $rsp['perms'];
			}
		}

		function auth_url( $frob, $perms = 'read' ) {
			$sig = md5( sprintf( '%sapi_key%sfrob%sperms%s', $this->secret, $this->api_key, $frob, $perms ) );
			return sprintf( 'https://flickr.com/services/auth/?api_key=%s&perms=%s&frob=%s&api_sig=%s', $this->api_key, $perms, $frob, $sig );
		}

		/*
				 * *****************************

		  To use the phpFlickr::call method, pass a string containing the API method you want
		  to use and an associative array of arguments.  For example:
		  $result = $f->call("flickr.photos.comments.getList", array("photo_id"=>'34952612'));
		  This method will allow you to make calls to arbitrary methods that haven't been
		  implemented in phpFlickr yet.

		 * ***************************** */

		function call( $method, $arguments ) {
			foreach ( $arguments as $key => $value ) {
				if ( is_null( $value ) ) {
					unset( $arguments[ $key ] );
				}
			}
			$this->request( $method, $arguments );
			return $this->parsed_response ? $this->parsed_response : false;
		}


		/* Authentication methods */

		function auth_checkToken() {
			/* https://www.flickr.com/services/api/flickr.auth.checkToken.html */
			$this->request( 'flickr.auth.checkToken' );
			return $this->parsed_response ? $this->parsed_response['auth'] : false;
		}



		/* People methods */

		function people_getInfo( $user_id ) {
			/* https://www.flickr.com/services/api/flickr.people.getInfo.html */
			$this->request( 'flickr.people.getInfo', array( 'user_id' => $user_id ) );
			return $this->parsed_response ? $this->parsed_response['person'] : false;
		}

		function people_getPhotos( $user_id, $args = array() ) {
			/*
			 This function strays from the method of arguments that I've
			 * used in the other functions for the fact that there are just
			 * so many arguments to this API method. What you'll need to do
			 * is pass an associative array to the function containing the
			 * arguments you want to pass to the API.  For example:
			 *   $photos = $f->photos_search(array("tags"=>"brown,cow", "tag_mode"=>"any"));
			 * This will return photos tagged with either "brown" or "cow"
			 * or both. See the API documentation (link below) for a full
			 * list of arguments.
			 */

			/* https://www.flickr.com/services/api/flickr.people.getPhotos.html */
			return $this->call( 'flickr.people.getPhotos', array_merge( array( 'user_id' => $user_id ), $args ) );
		}


		function people_getPublicPhotos( $user_id, $safe_search = null, $extras = null, $per_page = null, $page = null ) {
			/* https://www.flickr.com/services/api/flickr.people.getPublicPhotos.html */
			return $this->call(
				'flickr.people.getPublicPhotos',
				array(
					'user_id'     => $user_id,
					'safe_search' => $safe_search,
					'extras'      => $extras,
					'per_page'    => $per_page,
					'page'        => $page,
				)
			);
		}

		function photos_getInfo( $photo_id, $secret = null, $humandates = null, $privacy_filter = null, $get_contexts = null ) {
			/* https://www.flickr.com/services/api/flickr.photos.getInfo.html */
			return $this->call(
				'flickr.photos.getInfo',
				array(
					'photo_id'       => $photo_id,
					'secret'         => $secret,
					'humandates'     => $humandates,
					'privacy_filter' => $privacy_filter,
					'get_contexts'   => $get_contexts,
				)
			);
		}

	}

}
